dofile(ModPath .. "lua/core/sydchSO.lua")

function CriminalsManager:update_character_visual_state(character_name, visual_state)
	local character = self:character_by_name(character_name)

	if not character or not character.taken or not alive(character.unit) then
		return
	end

	visual_state = visual_state or {}
	local current_level = managers.job and managers.job:current_level_id()																								  
	local unit = character.unit
	local is_local_peer = visual_state.is_local_peer or character.visual_state.is_local_peer or false
	local visual_seed = visual_state.visual_seed or character.visual_state.visual_seed or CriminalsManager.get_new_visual_seed()
	local mask_id = visual_state.mask_id or character.visual_state.mask_id
	local armor_id = visual_state.armor_id or character.visual_state.armor_id or "level_1"
	local armor_skin = visual_state.armor_skin or character.visual_state.armor_skin or "none"
	local deployable_id = false

	if not is_local_peer then
		if visual_state.deployable_id ~= nil then
			deployable_id = visual_state.deployable_id
		elseif character.visual_state.deployable_id then
			deployable_id = character.visual_state.deployable_id
		end
	end

	local player_style = self:active_player_style() or managers.blackmarket:get_default_player_style()
	local suit_variation = nil
	local user_player_style = visual_state.player_style or character.visual_state.player_style or managers.blackmarket:get_default_player_style()

	if not self:is_active_player_style_locked() and user_player_style ~= managers.blackmarket:get_default_player_style() then
		if current_level and tweak_data.levels[current_level].player_style then
			player_style = tweak_data.levels[current_level] and tweak_data.levels[current_level].player_style
		else													 
			player_style = user_player_style
			suit_variation = visual_state.suit_variation or character.visual_state.suit_variation or "default"
		end
	end

	local glove_id = visual_state.glove_id or character.visual_state.glove_id or managers.blackmarket:get_default_glove_id()
	local character_visual_state = {
		is_local_peer = is_local_peer,
		visual_seed = visual_seed,
		player_style = player_style,
		suit_variation = suit_variation,
		glove_id = glove_id,
		mask_id = mask_id,
		armor_id = armor_id,
		armor_skin = armor_skin,
		deployable_id = deployable_id
	}

	local function get_value_string(value)
		return is_local_peer and tostring(value) or "third_" .. tostring(value)
	end

	if player_style then
		local unit_name = tweak_data.blackmarket:get_player_style_value(player_style, character_name, get_value_string("unit"))

		if unit_name then
			self:safe_load_asset(character, unit_name, "player_style")
		end
	end

	if glove_id then
		local unit_name = tweak_data.blackmarket:get_glove_value(glove_id, character_name, "unit", player_style, suit_variation)

		if unit_name then
			self:safe_load_asset(character, unit_name, "glove_id")
		end
	end

	if deployable_id and not is_local_peer then
		local deployable_tweak_data = tweak_data.equipments[deployable_id]
		local style_name = deployable_tweak_data.visual_style

		if style_name then
			local unit_name = tweak_data.blackmarket:get_player_style_value(style_name, character_name, get_value_string("unit"))

			if unit_name then
				self:safe_load_asset(character, unit_name, "deployable_id")
			end
		end
	end

	CriminalsManager.set_character_visual_state(unit, character_name, character_visual_state)

	character.visual_state = {
		is_local_peer = is_local_peer,
		visual_seed = visual_seed,
		player_style = user_player_style,
		suit_variation = suit_variation,
		glove_id = glove_id,
		mask_id = mask_id,
		armor_id = armor_id,
		armor_skin = armor_skin,
		deployable_id = deployable_id
	}
end

if not Network:is_server() or not sydchSO.settings.expandedCREW == 3 then
	return
end

-- AI IMPROVEMENTS: use peer colors, dodge preset, health setting etc.
local p = pairs
local is_offline = Global and Global.game_settings and Global.game_settings.single_player
local is_server = Network:is_server()
local total_chars = CriminalsManager.get_num_characters()
CriminalsManager.MAX_NR_TEAM_AI = total_chars or CriminalsManager.MAX_NR_TEAM_AI
local char_tweak = tweak_data.character

if char_tweak then
	local char_preset = char_tweak.presets
	if char_preset then
		local params = {
			dodge = { "poor", "average", "heavy", "athletic", "ninja" }
		}
		local char_damage = char_preset.gang_member_damage
		if char_damage then
			char_damage.HEALTH_INIT = 144
			char_damage.DOWNED_TIME = char_damage.DOWNED_TIME
		end
		local gang_weapon = char_preset.weapon.gang_member
		if gang_weapon then
			for k, v in p(gang_weapon) do
				v.focus_delay = 0
				v.aim_delay = { 0, 0 }
				v.range = deep_clone(char_preset.weapon.sniper.is_rifle.range)
				v.RELOAD_SPEED = 1
			end
			for k, v in p(char_tweak) do
				if type(v) == "table" then
					if v.access == "teamAI1" then
						v.no_run_start = true
						v.no_run_stop = true
						v.always_face_enemy = true
						v.damage.hurt_severity = char_preset.hurt_severities.only_light_hurt
						v.move_speed = is_server and char_preset.move_speed.lightning or v.move_speed
						v.dodge = char_preset.dodge[3]
						local orig = v.weapon.weapons_of_choice
						v.weapon = deep_clone(gang_weapon)
						v.weapon.weapons_of_choice = orig
					end
				end
			end
		end
	end
end

if is_offline or is_server then
	local old_color = CriminalsManager.character_color_id_by_unit
	function CriminalsManager:character_color_id_by_unit(unit, ...)
		local char_data = self:character_data_by_unit(unit)
		if char_data then
			if char_data.ai then
				if not char_data.ai_id then
					local ai_crim = self:nr_AI_criminals()
					char_data.ai_id = ai_crim + 1
					return char_data.ai_id
				end
				return char_data.ai_id
			end
		end
		return old_color(self, unit, ...)
	end
end