dofile(ModPath .. "lua/core/sydchSO.lua")

function HUDManager:add_teammate_panel(character_name, player_name, ai, peer_id)
	local function add_panel(i)
		self._teammate_panels[i]:add_panel()
		self._teammate_panels[i]:set_peer_id(peer_id)
		self._teammate_panels[i]:set_ai(ai)
		self:set_teammate_callsign(i, ai and tweak_data.max_players + 1 or peer_id)
		self:set_teammate_name(i, player_name)
		self:set_teammate_state(i, ai and "ai" or "player")

		local synced_cocaine_stacks = managers.player:get_synced_cocaine_stacks(peer_id)

		if synced_cocaine_stacks and synced_cocaine_stacks.in_use and not managers.player:has_category_upgrade("player", "armor_health_store_amount") then
			self:set_info_meter(i, {
				icon = "guis/dlcs/coco/textures/pd2/hud_absorb_stack_icon_01",
				max = 1,
				current = managers.player:get_peer_cocaine_damage_absorption_ratio(peer_id),
				total = managers.player:get_peer_cocaine_damage_absorption_max_ratio(peer_id)
			})
		end

		if peer_id then
			local peer_equipment = managers.player:get_synced_equipment_possession(peer_id) or {}

			for equipment, amount in pairs(peer_equipment) do
				self:add_teammate_special_equipment(i, {
					id = equipment,
					icon = tweak_data.equipments.specials[equipment].icon,
					amount = amount
				})
			end

			local peer_deployable_equipment = managers.player:get_synced_deployable_equipment(peer_id)

			if peer_deployable_equipment then
				local icon = tweak_data.equipments[peer_deployable_equipment.deployable].icon

				self:set_deployable_equipment(i, {
					icon = icon,
					amount = peer_deployable_equipment.amount
				})
			end

			local peer_cable_ties = managers.player:get_synced_cable_ties(peer_id)

			if peer_cable_ties then
				local icon = tweak_data.equipments.specials.cable_tie.icon

				self:set_cable_tie(i, {
					icon = icon,
					amount = peer_cable_ties.amount
				})
			end

			local peer_grenades = managers.player:get_synced_grenades(peer_id)

			if peer_grenades then
				local grenades_tweak = tweak_data.blackmarket.projectiles[peer_grenades.grenade]
				local icon = grenades_tweak.icon

				self:set_teammate_grenades(i, {
					icon = icon,
					amount = Application:digest_value(peer_grenades.amount, false)
				})
			end
		end

		local unit = managers.criminals:character_unit_by_name(character_name)

		if alive(unit) then
			local weapon = unit:inventory():equipped_unit()

			if alive(weapon) then
				local icon = weapon:base():weapon_tweak_data().hud_icon
				local equipped_selection = unit:inventory():equipped_selection()

				self:_set_teammate_weapon_selected(i, equipped_selection, icon)
			end
		end

		local peer_ammo_info = managers.player:get_synced_ammo_info(peer_id)

		if peer_ammo_info then
			for selection_index, ammo_info in pairs(peer_ammo_info) do
				self:set_teammate_ammo_amount(i, selection_index, unpack(ammo_info))
			end
		end

		local peer_carry_data = managers.player:get_synced_carry(peer_id)

		if peer_carry_data then
			self:set_teammate_carry_info(i, peer_carry_data.carry_id, managers.loot:get_real_value(peer_carry_data.carry_id, peer_carry_data.multiplier))
		end

		self._hud.teammate_panels_data[i].taken = true

		return i
	end

	local index = self._waiting_index[peer_id]

	if index and not self._hud.teammate_panels_data[index].taken then
		add_panel(index)
	end

	if self._waiting_index and peer_id then
		self._waiting_index[peer_id] = nil
	end

	for i, data in ipairs(self._hud.teammate_panels_data) do
		if not data.taken then
			return add_panel(i)
		end
	end
end

if sydchSO.settings.buffEFFECTS == 1 or sydchSO.settings.buffEFFECTS == 2 then
	return
end

local function skill_buff_effect_update(t, dt)
	local hud = managers.hud:script( PlayerBase.PLAYER_INFO_HUD_FULLSCREEN_PD2)
	local skill_buff_effect_hud_panel = hud.panel:child("skill_buff_effect_hud_panel")
	if not skill_buff_effect_hud_panel then
		return
	end
	
	if skill_buff_effect_effect_timer > 0 then
		skill_buff_effect_effect_timer = skill_buff_effect_effect_timer - TimerManager:main():delta_time()
		skill_buff_effect_hud_panel:set_visible(true)
		local hudinfo = managers.hud:script(PlayerBase.PLAYER_INFO_HUD_PD2)
		skill_buff_effect_hud_panel:animate(hudinfo.flash_icon, 4000000000)
	elseif skill_buff_effect_effect_timer <= 0 then
		skill_buff_effect_hud_panel:stop()
		skill_buff_effect_hud_panel:set_visible(false)
	end
end

Hooks:PostHook(HUDManager, "update", "sydchSO", function(self, t, dt)
	skill_buff_effect_update(t, dt)
end)

Hooks:PostHook(HUDManager, "activate_teammate_ability_radial", "sydchSO"..Idstring("deck_buffEFFECTS"):key(), function(self, buffEFFECTS_value, buffEFFECTS_time, buffEFFECTS_total)
	if type(buffEFFECTS_value) == type(HUDManager.PLAYER_PANEL) and buffEFFECTS_value == HUDManager.PLAYER_PANEL and type(buffEFFECTS_time) == "number" then
		local texture = "guis/textures/sso/alphawipe"
		local hud = managers.hud:script(PlayerBase.PLAYER_INFO_HUD_FULLSCREEN_PD2)
		if not hud.panel:child("deck_buff_effect") then
			local deck_buff_effect = hud.panel:bitmap({
				name = "deck_buff_effect",
				visible = false,
				texture = texture,
				layer = 0,
				alpha = 0.2,
				color = Color(0.8, 0.8, 0.8),
				blend_mode = "add",
				w = hud.panel:w(),
				h = hud.panel:h(),
				x = 0,
				y = 0 
			})
		end
		local deck_buff_effect = hud.panel:child("deck_buff_effect")
		if deck_buff_effect then
			local hudinfo = managers.hud:script(PlayerBase.PLAYER_INFO_HUD_PD2)
			local function anim(o)
				deck_buff_effect:set_visible(true)
				over(buffEFFECTS_time, function (p)
				
				end)
				deck_buff_effect:stop()
				deck_buff_effect:set_visible(false)
			end
			deck_buff_effect:stop()
			deck_buff_effect:animate(hudinfo.flash_icon, buffEFFECTS_time)
			deck_buff_effect:animate(anim)
		end
	end
end)

Hooks:PostHook(HUDManager, "set_player_health", "sydchSO", function(...)
	local hud = managers.hud:script(PlayerBase.PLAYER_INFO_HUD_FULLSCREEN_PD2)
	if not hud.panel:child("berserker_left") then
		local berserker_left = hud.panel:bitmap({
			name = "berserker_left",
			visible = false,
			texture = "guis/textures/sso/alphawipe",
			layer = 0,
			color = Color(0.8, 0.8, 0.8),
			alpha = 0,
			blend_mode = "add",
			w = hud.panel:w(),
			h = hud.panel:h(),
			x = 0,
			y = 0 
		})
	end
end)

local set_teammate_custom_radial_original = HUDManager.set_teammate_custom_radial
local set_teammate_ability_radial_original = HUDManager.set_teammate_ability_radial

function HUDManager:set_teammate_custom_radial(i, data, ...)
	local hud = managers.hud:script(PlayerBase.PLAYER_INFO_HUD_FULLSCREEN_PD2)
	local swan_song_left = hud.panel:child("swan_song_left")
	if not alive(swan_song_left) then
		swan_song_left = hud.panel:bitmap({
			name = "swan_song_left",
			visible = false,
			texture = "guis/textures/sso/alphawipe",
			layer = 0,
			color = Color(0.8, 0.8, 0.8),
			blend_mode = "add",
			w = hud.panel:w(),
			h = hud.panel:h(),
			x = 0,
			y = 0
		})
	end
	if i == HUDManager.PLAYER_PANEL and alive(swan_song_left) then
		if data.current < data.total and data.current > 0 then
			swan_song_left:set_visible(true)
			swan_song_left:set_alpha(0.2)
			local hudinfo = managers.hud:script(PlayerBase.PLAYER_INFO_HUD_PD2)
			swan_song_left:animate(hudinfo.flash_icon, 4000000000)
		else
			swan_song_left:stop()
			swan_song_left:set_visible(false)
		end
	end
	return set_teammate_custom_radial_original(self, i, data, ...)
end

function HUDManager:set_teammate_ability_radial(i, data, ...)
	local hud = managers.hud:script(PlayerBase.PLAYER_INFO_HUD_FULLSCREEN_PD2)
	local chico_injector_left = hud.panel:child("chico_injector_left")
	if not alive(chico_injector_left) then
		chico_injector_left = hud.panel:bitmap({
			name = "chico_injector_left",
			visible = false,
			texture = "guis/textures/sso/alphawipe",
			layer = 0,
			alpha = 0.1,
			color = Color(0.8, 0.8, 0.8),
			blend_mode = "add",
			w = hud.panel:w(),
			h = hud.panel:h(),
			x = 0,
			y = 0
		})
	end
	if i == HUDManager.PLAYER_PANEL and alive(chico_injector_left) then
		if data.current < data.total and data.current > 0 then
			chico_injector_left:set_visible(true)
			chico_injector_left:set_alpha(0.1)
			local hudinfo = managers.hud:script(PlayerBase.PLAYER_INFO_HUD_PD2)
			chico_injector_left:animate(hudinfo.flash_icon, 4000000000)
		else
			chico_injector_left:stop()
			chico_injector_left:set_visible(false)
		end
	end
	return set_teammate_ability_radial_original(self, i, data, ...)
end