dofile(ModPath .. "lua/core/sydchSO.lua")

local _get_melee_weapon_stats = BlackMarketGui._get_melee_weapon_stats
function BlackMarketGui:_get_melee_weapon_stats(name)
    local base_stats, mods_stats, skill_stats = _get_melee_weapon_stats(self, name)
    local base_charge_time = base_stats.charge_time.value
    local multiplier = managers.player:upgrade_value("player", "melee_swing_multiplier_delay", 1)

    if base_charge_time > 0 and multiplier ~= 1 then
        skill_stats.charge_time = {value = (base_charge_time * multiplier) - base_charge_time, skill_in_effect = true}
    end

    return base_stats, mods_stats, skill_stats
end

function BlackMarketGui:populate_armors(data)
    local new_data = {}
    local sort_data = {}
    for id, d in pairs(Global.blackmarket_manager.armors) do
        table.insert(sort_data, id)
    end
    local armor_level_data = {}
    for level, data in pairs(tweak_data.upgrades.level_tree) do
        if data.upgrades then
            for _, upgrade in ipairs(data.upgrades) do
                local def = tweak_data.upgrades.definitions[upgrade]
                if def.armor_id then
                    armor_level_data[def.armor_id] = level
                end
            end
        end
    end
    table.sort(sort_data, function(x, y)
        local x_level = tonumber(string.sub(x, 7))
        local y_level = tonumber(string.sub(y, 7))
        return x_level < y_level
    end)
    local guis_catalog = "guis/"
    local index = 0
    for i, armor_id in ipairs(sort_data) do
        local name_id = tweak_data.blackmarket.armors[armor_id] and tweak_data.blackmarket.armors[armor_id].name_id or ""
        local bm_data = Global.blackmarket_manager.armors[armor_id]
        guis_catalog = "guis/"
        local bundle_folder = tweak_data.blackmarket.armors[armor_id] and tweak_data.blackmarket.armors[armor_id].texture_bundle_folder
        if bundle_folder then
            guis_catalog = guis_catalog .. "dlcs/" .. tostring(bundle_folder) .. "/"
        end
        index = index + 1
        new_data = {}
        new_data.name = armor_id
        new_data.name_localized = managers.localization:text(name_id)
        new_data.category = "armors"
        new_data.slot = index
        new_data.unlocked = bm_data.unlocked
        new_data.level = armor_level_data[armor_id] or 0
        new_data.skill_based = new_data.level == 0
        new_data.equipped = bm_data.equipped
        new_data.skill_name = new_data.level == 0 and "bm_menu_skill_locked_" .. new_data.name
        new_data.bitmap_texture = guis_catalog .. "textures/pd2/blackmarket/icons/armors/" .. new_data.name
        new_data.comparision_data = {}
        new_data.lock_texture = self:get_lock_icon(new_data)
        if i ~= 1 and managers.blackmarket:got_new_drop("normal", "armors", armor_id) then
            new_data.mini_icons = new_data.mini_icons or {}
            table.insert(new_data.mini_icons, {
                name = "new_drop",
                texture = "guis/textures/pd2/blackmarket/inv_newdrop",
                right = 0,
                top = 0,
                layer = 1,
                w = 16,
                h = 16,
                stream = false
            })
            new_data.new_drop_data = {
                "normal",
                "armors",
                armor_id
            }
        end
        if new_data.unlocked and not new_data.equipped then
            table.insert(new_data, "a_equip")
        end
        if i ~= 1 then
            if (game_state_machine and game_state_machine:current_state_name()) ~= "ingame_waiting_for_players" then
                table.insert(new_data, "a_mod")
            end
        end
        data[index] = new_data
    end
    local max_armors = data.override_slots[1] * data.override_slots[2]
    for i = 1, max_armors do
        if not data[i] then
            new_data = {}
            new_data.name = "empty"
            new_data.name_localized = ""
            new_data.category = "armors"
            new_data.slot = i
            new_data.unlocked = true
            new_data.equipped = false
            data[i] = new_data
        end
    end
end