dofile(ModPath .. "lua/core/sydchSO.lua")

if sydchSO.settings.betterMUTATORS == false then
	return
end

function MutatorsManager:init()
	managers.mutators = self
	self._message_system = MessageSystem:new()
	self._lobby_delay = -1
	if not Global.mutators then
		Global.mutators = {
			mutator_values = {},
			active_on_load = {}
		}
	end

	self._mutators = {
		MutatorEnemyHealth:new(self),
		MutatorEnemyDamage:new(self),
		MutatorFriendlyFire:new(self),
		MutatorShotgunTweak:new(self),
		MutatorExplodingEnemies:new(self),
		MutatorHydra:new(self),
		MutatorEnemyReplacer:new(self),
		MutatorMediDozer:new(self),
		MutatorCloakerEffect:new(self),
		MutatorShieldDozers:new(self),
		MutatorTitandozers:new(self),
		MutatorCG22:new(self),
		MutatorPiggyRevenge:new(self)
		-- -- -- -- -- -- -- -- -- -- --
		--MutatorBirthday:new(self)
		-- -- -- -- -- -- -- -- -- -- --
		--MutatorOneDown:new(self),
		--MutatorRealism:new(self),
		--MutatorOneFortyFiver:new(self),
		--MutatorWarCrimes:new(self),
		--MutatorRational:new(self),
		--MutatorDevilsSky:new(self),
		--MutatorItTakesOne:new(self),
		--MutatorCompetitive:new(self),
		--MutatorNotInTheFace:new(self),
		--MutatorCrabPeople:new(self),
		--MutatorDeathFromAbove:new(self),
		--MutatorHeistersMustDie:new(self),
		--MutatorPerfectScore:new(self),
		--MutatorRoughTalker:new(self),
		--MutatorCloudCamera:new(self)
	}
	
	self._active_mutators = {}
	local activate = Global.mutators and Global.mutators.active_on_load
	if not self:can_mutators_be_active() then
		activate = false
	end

	if activate then
		for id, data in pairs(Global.mutators.active_on_load) do
			cat_print("jamwil", "[Mutators] Attempting to activate mutator: ", id)

			local mutator = self:get_mutator_from_id(id)

			if mutator then
				table.insert(self:active_mutators(), {
					mutator = mutator
				})
				cat_print("jamwil", "[Mutators] Activated mutator: ", id)
			else
				cat_print("jamwil", "[Mutators] No mutator with id: ", id)
			end

			for key, value in pairs(data) do
				if Network:is_client() then
					mutator:set_host_value(key, value)
				end
			end
		end
	end

	local setup_mutators = {}
	for _, active_mutator in pairs(self:active_mutators()) do
		table.insert(setup_mutators, active_mutator.mutator)
	end

	table.sort(setup_mutators, function (a, b)
		return b.load_priority < a.load_priority
	end)

	for _, mutator in pairs(setup_mutators) do
		cat_print("jamwil", "[Mutators] Setting up active mutator: ", mutator:id())
		mutator:setup(self)
	end
end

function MutatorsManager:are_challenges_disabled()
	if game_state_machine:current_state_name() ~= "menu_main" then
		for _, mutator in pairs(self:mutators()) do
			if mutator:is_active() and mutator.disables_challenges then
				return true
			end
		end
	else
		return false
	end
end

function MutatorsManager:are_trophies_disabled()
	if game_state_machine:current_state_name() ~= "menu_main" then
		for _, mutator in pairs(self:mutators()) do
			if mutator:is_active() and mutator.disables_trophies then
				return true
			end
		end
	else
		return false
	end
end

function MutatorsManager:should_disable_statistics()
	if game_state_machine:current_state_name() ~= "menu_main" then
		for _, mutator in pairs(self:mutators()) do
			if mutator:is_active() and mutator.disables_statistics then
				return true
			end
		end
	else
		return false
	end
end

function MutatorsManager:_get_reduction(func)
	local max_reduction = 0

	for _, mutator in pairs(self:mutators()) do
		if mutator:is_enabled() or mutator:is_active() then
			max_reduction = math.max(max_reduction, mutator[func](mutator))
		end
	end

	return max_reduction
end

function MutatorsManager:get_cash_multiplier()
	local value = self:_get_reduction("get_cash_reduction")
	if value < 0.1 then
		return 1
	else
		return 1 * value
	end
end

function MutatorsManager:get_cash_reduction()
	return self:_get_reduction("get_cash_reduction")
end

function MutatorsManager:get_experience_multiplier()
	local value = self:_get_reduction("get_experience_reduction") 
		if value < 0.1 then
		return 1
	else
		return 1 * self:_get_reduction("get_experience_reduction")
	end
end

function MutatorsManager:get_experience_reduction()
	return self:_get_reduction("get_experience_reduction")
end