dofile(ModPath .. "lua/core/sydchSO.lua")

local mvec3_set = mvector3.set
local mvec3_add = mvector3.add
local mvec3_dot = mvector3.dot
local mvec3_sub = mvector3.subtract
local mvec3_mul = mvector3.multiply
local mvec3_norm = mvector3.normalize
local mvec3_dir = mvector3.direction
local mvec3_set_l = mvector3.set_length
local mvec3_len = mvector3.length
local math_clamp = math.clamp
local math_lerp = math.lerp
local math_map_range_clamped = math.map_range_clamped
local tmp_vec1 = Vector3()
local tmp_vec2 = Vector3()
local ids_single = Idstring("single")
local ids_auto = Idstring("auto")
local ids_burst = Idstring("burst")
local ids_volley = Idstring("volley")
local FIRE_MODE_IDS = {
	single = ids_single,
	auto = ids_auto,
	burst = ids_burst,
	volley = ids_volley
}

NewRaycastWeaponBase = NewRaycastWeaponBase or class(RaycastWeaponBase)

Hooks:PostHook(NewRaycastWeaponBase, "fire", "sydchSO", function(self)
	if managers.player:has_category_upgrade("weapon", "automatic_head_shot_add") then
		local expertise_chance = 0.3
		if math.random() <= expertise_chance then
			self._can_shoot_through_enemy = true
			self._can_shoot_through_wall = true
			expertise_chance = 0.3
		else
			self._can_shoot_through_enemy = false
			self._can_shoot_through_wall = false
			expertise_chance = expertise_chance + 0.1
		end
	end
	if managers.player:has_activate_temporary_upgrade("temporary", "chico_injector") or managers.player:has_activate_temporary_upgrade("temporary", "copr_risen") then
		self:set_ammo_remaining_in_clip(math.min(self:get_ammo_remaining_in_clip() + 1))
	end
	local clip = self:ammo_base():get_ammo_remaining_in_clip()
	if managers.player:has_category_upgrade("pistol", "last_bullet_refill_chance") and self:is_category("pistol") and 2 > clip and math.random() <= 0.3 then
		self:set_ammo_remaining_in_clip(math.min(self:get_ammo_remaining_in_clip() + 5))
		self:set_ammo_total(math.min(self:get_ammo_total() + 5))
	end
end)

function NewRaycastWeaponBase:recoil_multiplier()
	local is_moving = false
	local user_unit = self._setup and self._setup.user_unit
	local stamina_ratio =  1 - (self._setup.user_unit:movement():stamina() / tweak_data.player.movement_state.stamina.STAMINA_INIT)
	local suppression_ratio =  1 + (managers.player:player_unit():character_damage():effective_suppression_ratio())

	if user_unit then
		is_moving = alive(user_unit) and user_unit:movement() and user_unit:movement()._current_state and user_unit:movement()._current_state._moving and stamina_ratio and suppression_ratio
	end

	return managers.blackmarket:recoil_multiplier(self._name_id, self:weapon_tweak_data().categories, self._silencer, self._blueprint, is_moving) and stamina_ratio and suppression_ratio
end

function NewRaycastWeaponBase:conditional_accuracy_addend(current_state)
	local index = 0

	if not current_state then
		return index
	end

	local pm = managers.player

	if not current_state:in_steelsight() then
		index = index + pm:upgrade_value("player", "hip_fire_accuracy_inc", 0) - 4
	end
	
	if sydchSO.settings.weaponOVERHAUL == true then
	if current_state:in_steelsight() then
		index = index + 2
	end

	if self:is_single_shot() then
		index = index + 2
	end
	end
	
	if self:is_single_shot() and self:is_category("assault_rifle", "smg", "snp") then
		index = index + pm:upgrade_value("weapon", "single_spread_index_addend", 0)
	elseif not self:is_single_shot() then
		index = index + pm:upgrade_value("weapon", "auto_spread_index_addend", 0)
	end

	if not current_state._moving then
		index = index + pm:upgrade_value("player", "not_moving_accuracy_increase", 0) + 1
	end

	if current_state._moving then
		for _, category in ipairs(self:categories()) do
			index = index + pm:upgrade_value(category, "move_spread_index_addend", 0) - 2
		end
	end
	
	-- jonaske's math
	index = index / 2 * (1 + self._setup.user_unit:movement():stamina() / tweak_data.player.movement_state.stamina.STAMINA_INIT)
	index = 2 * index / (1 + pm:player_unit():character_damage():effective_suppression_ratio())

	return index
end

function NewRaycastWeaponBase:conditional_accuracy_multiplier(current_state)
	local mul = 1

	if not current_state then
		return mul
	end

	local pm = managers.player

	if sydchSO.settings.weaponOVERHAUL == true then
	if current_state:in_steelsight() and self:is_single_shot() then
		mul = mul + 1 - pm:upgrade_value("player", "single_shot_accuracy_inc", 1)
	end

	if current_state:in_steelsight() then
		for _, category in ipairs(self:categories()) do
			mul = mul + 1 - managers.player:upgrade_value(category, "steelsight_accuracy_inc", 1)
		end
	end
	end

	if current_state:in_steelsight() and current_state._moving then
		mul = mul + 1 - pm:upgrade_value("player", "weapon_movement_stability", 1)
	end

	mul = mul + 1 - pm:get_property("desperado", 1)
	
	-- jonaske's math
	mul = mul / 2 * (1 + self._setup.user_unit:movement():stamina() / tweak_data.player.movement_state.stamina.STAMINA_INIT)
	mul = 2 * mul / (1 + pm:player_unit():character_damage():effective_suppression_ratio())

	return self:_convert_add_to_mul(mul)
end

function NewRaycastWeaponBase:get_add_head_shot_mul()
	if self:is_category("smg", "lmg", "assault_rifle", "minigun") or self:is_category("saw", "flamethrower") then
		return managers.player:upgrade_value("weapon", "automatic_head_shot_add", nil)
	end

	return nil
end

function NewRaycastWeaponBase:reload_speed_multiplier()
	if self._current_reload_speed_multiplier then
		return self._current_reload_speed_multiplier
	end

	local multiplier = 1

	for _, category in ipairs(self:weapon_tweak_data().categories) do
		multiplier = multiplier + 1 - managers.player:upgrade_value(category, "reload_speed_multiplier", 1)
	end

	multiplier = multiplier + 1 - managers.player:upgrade_value("weapon", "passive_reload_speed_multiplier", 1)
	multiplier = multiplier + 1 - managers.player:upgrade_value(self._name_id, "reload_speed_multiplier", 1)

	if self._setup and alive(self._setup.user_unit) and self._setup.user_unit:movement() then
		local morale_boost_bonus = self._setup.user_unit:movement():morale_boost()

		if morale_boost_bonus then
			multiplier = multiplier + 1 - morale_boost_bonus.reload_speed_bonus
		end

		if self._setup.user_unit:movement():next_reload_speed_multiplier() then
			multiplier = multiplier + 1 - self._setup.user_unit:movement():next_reload_speed_multiplier()
		end
	end

	if managers.player:has_activate_temporary_upgrade("temporary", "reload_weapon_faster") then
		multiplier = multiplier + 1 - managers.player:temporary_upgrade_value("temporary", "reload_weapon_faster", 1)
	end
	
	if managers.player:has_activate_temporary_upgrade("temporary", "combat_medic_reload_speed_multiplier") then
		multiplier = multiplier + 1 - managers.player:temporary_upgrade_value("temporary", "combat_medic_reload_speed_multiplier", 1)
	end

	if managers.player:has_activate_temporary_upgrade("temporary", "gun_kata_onenemykilled") then
		multiplier = multiplier + 1 - managers.player:temporary_upgrade_value("temporary", "gun_kata_onenemykilled", 1)
	end
	
	if managers.player:has_activate_temporary_upgrade("temporary", "gun_kata_onplayerdodge") then
		multiplier = multiplier + 1 - managers.player:temporary_upgrade_value("temporary", "gun_kata_onplayerdodge", 1)
	end
	
	local ammo_base = self._reload_ammo_base or self:ammo_base()
	local clip = ammo_base:get_ammo_remaining_in_clip()
	local max = ammo_base:get_ammo_max_per_clip()
	
	multiplier = multiplier + 1 - managers.player:get_property("shock_and_awe_reload_multiplier", 1)
	multiplier = multiplier + 1 - managers.player:get_temporary_property("bloodthirst_reload_speed", 1)
	multiplier = multiplier + 1 - managers.player:upgrade_value("team", "crew_faster_reload", 1)
	multiplier = multiplier + 1 - managers.player:get_property("desperado", 1)
	multiplier = self:_convert_add_to_mul(multiplier)
	multiplier = multiplier * self:reload_speed_stat()
	multiplier = managers.modifiers:modify_value("WeaponBase:GetReloadSpeedMultiplier", multiplier)

	return multiplier
end

function NewRaycastWeaponBase:enter_steelsight_speed_multiplier()
	local multiplier = 1
	local categories = self:weapon_tweak_data().categories

	for _, category in ipairs(categories) do
		multiplier = multiplier + 1 - managers.player:upgrade_value(category, "enter_steelsight_speed_multiplier", 1)
	end

	multiplier = multiplier + 1 - managers.player:temporary_upgrade_value("temporary", "combat_medic_enter_steelsight_speed_multiplier", 1)
	multiplier = multiplier + 1 - managers.player:upgrade_value(self._name_id, "enter_steelsight_speed_multiplier", 1)
	multiplier = multiplier + 1 - managers.player:upgrade_value("weapon", "enter_steelsight_speed_multiplier", 1)
	multiplier = multiplier + 1 - managers.player:upgrade_value("team", "crew_aimer", 1)
	multiplier = multiplier + 1 - managers.player:temporary_upgrade_value("temporary", "steelsight_weapon_faster", 1)

	if self._silencer then
		multiplier = multiplier + 1 - managers.player:upgrade_value("weapon", "silencer_enter_steelsight_speed_multiplier", 1)

		for _, category in ipairs(categories) do
			multiplier = multiplier + 1 - managers.player:upgrade_value(category, "silencer_enter_steelsight_speed_multiplier", 1)
		end
	end
	
	if managers.player:has_activate_temporary_upgrade("temporary", "gun_kata_onenemykilled") then
		multiplier = multiplier * managers.player:temporary_upgrade_value("temporary", "gun_kata_onenemykilled", 1)
	end
	
	--if managers.player:has_activate_temporary_upgrade("temporary", "gun_kata_onplayerdodge") then
	--	multiplier = multiplier * managers.player:temporary_upgrade_value("temporary", "gun_kata_onplayerdodge", 1)
	--end

	return self:_convert_add_to_mul(multiplier)
end

function NewRaycastWeaponBase:fire_rate_multiplier()
	local multiplier = self._fire_rate_multiplier or 1
	
	--if managers.player:has_activate_temporary_upgrade("temporary", "gun_kata") then
	--	multiplier = multiplier * managers.player:temporary_upgrade_value("temporary", "gun_kata", 1)
	--end
	
	local clip = self:ammo_base():get_ammo_remaining_in_clip()
	local max = self:ammo_base():get_ammo_max_per_clip()
	if managers.player:has_category_upgrade("weapon", "half_magazine_fire_rate") and (max * 0.5) > clip then
		multiplier = multiplier * managers.player:temporary_upgrade_value("weapon", "half_magazine_fire_rate", 1)
	end
	multiplier = multiplier * managers.player:temporary_upgrade_value("temporary", "reload_fire_rate", 1)
	if self:is_single_shot() then
		multiplier = multiplier * managers.player:temporary_upgrade_value("weapon", "single_shot_fire_rate", 1)
	end

	return multiplier
end

local on_reload = NewRaycastWeaponBase.on_reload
function NewRaycastWeaponBase:on_reload(...)
	if self._setup.expend_ammo and Global.game_settings.one_down and not self:is_category("shell_load") and sydchSO.settings.restorePROJOB == true then
		local ammo_base = self._reload_ammo_base or self:ammo_base()
		ammo_base:set_ammo_total(ammo_base:get_ammo_total() - ammo_base:get_ammo_remaining_in_clip())
	end
	if sydchSO.settings.weaponOVERHAUL == true then
		local ammo_base = self._reload_ammo_base or self:ammo_base()
		
		if ammo_base:weapon_tweak_data().uses_clip == true then
			if ammo_base:get_ammo_remaining_in_clip() <= ammo_base:get_ammo_max_per_clip() then
				local clip_capacity = ammo_base:weapon_tweak_data().clip_capacity
				local total_ammo = ammo_base:get_ammo_total()
				local remaining_ammo = ammo_base:get_ammo_remaining_in_clip()
		
				ammo_base:set_ammo_remaining_in_clip(math.min(total_ammo, ammo_base:get_ammo_max_per_clip(), remaining_ammo + clip_capacity))
			end
		else
			local tactical_reload = ammo_base:weapon_tweak_data().tactical_reload
			local full_capacity = ammo_base:weapon_tweak_data().full_capacity or (ammo_base:get_ammo_max_per_clip() + 1)
			local remaining_in_clip = ammo_base:get_ammo_remaining_in_clip()
			local total_ammo = ammo_base:get_ammo_total()
			local max_per_clip = ammo_base:get_ammo_max_per_clip()
		
			if tactical_reload == 1 then
				if remaining_in_clip > 0 then
					if remaining_in_clip <= (full_capacity - tactical_reload) then
						local min_ammo = tactical_reload > 2 and remaining_in_clip + tactical_reload or max_per_clip + tactical_reload
						ammo_base:set_ammo_remaining_in_clip(math.min(total_ammo, min_ammo) - 1)
					else
						ammo_base:set_ammo_remaining_in_clip(math.min(total_ammo, full_capacity) - 1)
					end
				elseif remaining_in_clip == 0 then
					local min_ammo = tactical_reload > 2 and remaining_in_clip + tactical_reload or max_per_clip
					ammo_base:set_ammo_remaining_in_clip(math.min(total_ammo, min_ammo) - 1)
				end
			elseif remaining_in_clip > 0 then
				ammo_base:set_ammo_remaining_in_clip(math.min(total_ammo, max_per_clip))
			elseif self._setup.expend_ammo then
				ammo_base:set_ammo_remaining_in_clip(math.min(total_ammo, max_per_clip))
			else
				ammo_base:set_ammo_remaining_in_clip(max_per_clip)
				ammo_base:set_ammo_total(max_per_clip)
			end
		end

		managers.job:set_memory("kill_count_no_reload_" .. tostring(self._name_id), nil, true)
		self._reload_ammo_base = nil
	else
		on_reload(self, ...)
	end
end

if not sydchSO.settings.weaponOVERHAUL == true then
	return
end

function NewRaycastWeaponBase:clip_full()
    local ammo_base = self:ammo_base()
    local weapon_tweak_data = ammo_base:weapon_tweak_data()

    if weapon_tweak_data.tactical_reload and weapon_tweak_data.full_capacity then
        if weapon_tweak_data.full_capacity < ammo_base:get_ammo_max_per_clip() then
            weapon_tweak_data.full_capacity = ammo_base:get_ammo_max_per_clip()
        end
        return ammo_base:get_ammo_remaining_in_clip() == weapon_tweak_data.full_capacity
    elseif weapon_tweak_data.tactical_reload then
        local tactical_reload_amount = weapon_tweak_data.tactical_reload
        local expected_ammo = ammo_base:get_ammo_max_per_clip() + tactical_reload_amount
        return ammo_base:get_ammo_remaining_in_clip() == expected_ammo
    else
        return ammo_base:get_ammo_remaining_in_clip() == ammo_base:get_ammo_max_per_clip()
    end
end

function NewRaycastWeaponBase:reload_expire_t(is_not_empty)
	if self._use_shotgun_reload then
		local ammo_total = self:get_ammo_total()
		local ammo_max_per_clip = self:get_ammo_max_per_clip()
		local tactical_reload = self:weapon_tweak_data().tactical_reload
		local ammo_remaining_in_clip = self:get_ammo_remaining_in_clip()
		
		if ammo_remaining_in_clip > 0 and tactical_reload == 1 then
			local ammo_max_per_clip = ammo_max_per_clip + 1
		end
		
		local ammo_to_reload = math.min(ammo_total - ammo_remaining_in_clip, ammo_max_per_clip - ammo_remaining_in_clip)
		local shotgun_reload_tweak = self:_get_shotgun_reload_tweak_data(is_not_empty)

		if shotgun_reload_tweak and shotgun_reload_tweak.reload_queue then
			local reload_expire_t = 0
			local queue_index = 0
			local queue_data = nil
			local queue_num = #shotgun_reload_tweak.reload_queue

			while ammo_to_reload > 0 do
				if queue_index == queue_num then
					reload_expire_t = reload_expire_t + (shotgun_reload_tweak.reload_queue_wrap or 0)
				end

				queue_index = queue_index % queue_num + 1
				queue_data = shotgun_reload_tweak.reload_queue[queue_index]
				reload_expire_t = reload_expire_t + queue_data.expire_t or 0.5666666666666667
				ammo_to_reload = ammo_to_reload - (queue_data.reload_num or 1)
			end

			return reload_expire_t
		end

		local reload_shell_expire_t = self:reload_shell_expire_t(is_not_empty)
		local reload_num = shotgun_reload_tweak and shotgun_reload_tweak.reload_num or 1

		return math.ceil(ammo_to_reload / reload_num) * reload_shell_expire_t
	end

	return nil
end

function NewRaycastWeaponBase:update_reloading(t, dt, time_left)
	if self._use_shotgun_reload and self._next_shell_reloded_t and self._next_shell_reloded_t < t then
		local speed_multiplier = self:reload_speed_multiplier()
		local shotgun_reload_tweak = self:_get_shotgun_reload_tweak_data(not self._started_reload_empty)
		local ammo_to_reload = 1
		local next_queue_data = nil

		if shotgun_reload_tweak and shotgun_reload_tweak.reload_queue then
			self._shotgun_queue_index = self._shotgun_queue_index % #shotgun_reload_tweak.reload_queue + 1

			if self._shotgun_queue_index == #shotgun_reload_tweak.reload_queue then
				self._next_shell_reloded_t = self._next_shell_reloded_t + (shotgun_reload_tweak.reload_queue_wrap or 0)
			end

			local queue_data = shotgun_reload_tweak.reload_queue[self._shotgun_queue_index]
			ammo_to_reload = queue_data and queue_data.reload_num or 1
			next_queue_data = shotgun_reload_tweak.reload_queue[self._shotgun_queue_index + 1]
			self._next_shell_reloded_t = self._next_shell_reloded_t + (next_queue_data and next_queue_data.expire_t or 0.5666666666666667) / speed_multiplier
		else
			self._next_shell_reloded_t = self._next_shell_reloded_t + self:reload_shell_expire_t(not self._started_reload_empty) / speed_multiplier
			ammo_to_reload = shotgun_reload_tweak and shotgun_reload_tweak.reload_num or 1
		end

		if self:get_ammo_remaining_in_clip() > 0 and self:weapon_tweak_data().tactical_reload == 1 then
			self:set_ammo_remaining_in_clip(math.min(self:get_ammo_total(), self:get_ammo_max_per_clip() + 1, self:get_ammo_remaining_in_clip() + ammo_to_reload))
		else
			self:set_ammo_remaining_in_clip(math.min(self:get_ammo_total(), self:get_ammo_max_per_clip(), self:get_ammo_remaining_in_clip() + ammo_to_reload))
		end
		managers.job:set_memory("kill_count_no_reload_" .. tostring(self._name_id), nil, true)

		if not next_queue_data or not next_queue_data.skip_update_ammo then
			self:update_ammo_objects()
		end

		return true
	end
end