if sydchSO and sydchSO.CopUtils then
    return
end

dofile(ModPath .. "lua/core/sydchSO.lua")

sydchSO.CopUtils = {}
sydchSO.CopUtils._arrest_search_radius = 900
sydchSO.CopUtils._arrest_action_radius = 100
sydchSO.CopUtils._minimum_interact_time = 0.35

function sydchSO.CopUtils:CheckLocalMeleeDamageArrest(player_unit, attacker_unit, is_melee)
    if player_unit ~= managers.player:player_unit() then
        return nil, "not local player unit"
    end

    local state = player_unit:movement():current_state()
    local is_interacting = state._interacting and state:_interacting()
    if not is_interacting then
        return false, "not interacting"
    end

    if not state._interact_params or not state._interact_params.timer or not state._interact_expire_t then
        return false, "interaction params invalid"
    end

    local current_interact_t = state._interact_params.timer - state._interact_expire_t
    if current_interact_t < self._minimum_interact_time then
        return false, "interaction too short"
    end

    if not is_melee and attacker_unit and alive(attacker_unit) then
        local dist = mvector3.distance(player_unit:position(), attacker_unit:position())
        if dist > self._arrest_action_radius then
            return false, "too far away for non-melee arrest"
        end
    end

    return "arrested"
end

function sydchSO.CopUtils:CheckClientMeleeDamageArrest(player_unit, attacker_unit, is_melee)
    if Network and Network:is_client() then
        return nil, "not host, no husk check"
    end

    if not player_unit or not player_unit.alive or not player_unit:alive() or not player_unit.movement or not player_unit:movement() or not player_unit:movement()._interaction_tweak then
        return false, "husk not interacting"
    end

    if not is_melee and attacker_unit and alive(attacker_unit) then
        local dist = mvector3.distance(player_unit:position(), attacker_unit:position())
        if dist > self._arrest_action_radius then
            return false, "cop too far away from husk"
        end
    end

    return "arrested"
end

function sydchSO.CopUtils:CounterArrestAttacker(player_unit, attacker_unit)
    return self:KnockDownAttacker(player_unit, attacker_unit)
end

function sydchSO.CopUtils:KnockDownAttacker(player_unit, attacker_unit)
    local action_data = {
        damage_effect = 1,
        damage = 0,
        variant = "counter_spooc",
        attacker_unit = player_unit,
        col_ray = {
            body = attacker_unit:body("body"),
            position = attacker_unit:position() + math.UP * 100
        },
        name_id = managers.blackmarket:equipped_melee_weapon(),
        attack_dir = Vector3(0,1,0)
    }

    attacker_unit:character_damage():damage_melee(action_data)
end

function sydchSO.CopUtils:SendCopToArrestPlayer(player_unit)
    if Network and Network:is_client() then
        return
    end

    if managers.groupai:state():whisper_mode() then
        return
    end

    if not player_unit or not player_unit.alive or not player_unit:alive() then
        return
    end

    local enemies = World:find_units_quick(player_unit, "sphere", player_unit:position(), self._arrest_search_radius, managers.slot:get_mask("enemies"))
    if not enemies or #enemies <= 0 then
        return
    end

    local lowest_distance = 999999
    local closest_enemy = nil
    local highest_found_priority = -100
    local playerpos = player_unit:position()

    local objective = {
        type = "free",
        haste = "run",
        pose = "stand",
        nav_seg = managers.navigation:get_nav_seg_from_pos(player_unit:position(), true),
        pos = mvector3.copy(player_unit:position()),
        forced = true,
        important = true
    }

    for i, enemy in pairs(enemies) do
        if self:AreUnitsEnemies(player_unit, enemy) then
            local enemy_chartweak = enemy:base():char_tweak()
            local prio = enemy_chartweak.arrest_player_priority or -10

            if enemy_chartweak.access ~= "gangster" and (enemy_chartweak.arrest_player_priority or not managers.groupai:state():get_assault_mode()) then
                local dist = mvector3.distance(enemy:position(), playerpos)
                local is_available = enemy:brain():is_available_for_assignment(objective)

                if prio > highest_found_priority and is_available then
                    lowest_distance = dist
                    highest_found_priority = prio
                    closest_enemy = enemy
                elseif prio == highest_found_priority and dist < lowest_distance and is_available then
                    lowest_distance = dist
                    highest_found_priority = prio
                    closest_enemy = enemy
                end

            end
        end
    end

    if closest_enemy then
        objective = {
            type = "free",
            haste = "run",
            pose = "stand",
            nav_seg = managers.navigation:get_nav_seg_from_pos(player_unit:position(), true),
            pos = mvector3.copy(player_unit:position()),
            complete_clbk = callback(self, self, '_onCopArrivedAtArrestPosition', {cop = closest_enemy, target = player_unit}),
            forced = true,
            important = true
        }
        closest_enemy:brain():set_objective(objective)
        closest_enemy:brain():set_logic("travel")
        closest_enemy:movement():action_request({
            type = "idle",
            body_part = 1,
            sync = true
        })
    end
end

function sydchSO.CopUtils:_onCopArrivedAtArrestPosition(clbk_data)
    if Network and Network:is_client() then
        return
    end

    local cop = clbk_data.cop
    local player_unit = clbk_data.target

    if not alive(cop) or not alive(player_unit) then
        log("Cop arrived at arrest position but there was no cop or player set")
        return
    end

    local result = nil
    if player_unit == managers.player:player_unit() then
        result = self:CheckLocalMeleeDamageArrest(player_unit, cop)
    else
        local peer = managers.network:session():peer_by_unit(player_unit)
        if peer and (sydchSO.peersWithMod[peer:id()] or sydchSO.arrestModPeers[peer:id()]) then
            return self:TellClientCheckArrest(peer:id(), cop:id())
        end
        result = self:CheckClientMeleeDamageArrest(player_unit, cop)
    end

    if result == "arrested" then
        player_unit:movement():on_cuffed()
        cop:sound():say("i03", true, false)
    elseif result == "counterarrest" then
        self:CounterArrestAttacker(player_unit, cop)
    elseif result == "countered" then
        self:KnockDownAttacker(player_unit, cop)
    end
end

function sydchSO.CopUtils:TellClientCheckArrest(peer_id, cop_id)
    LuaNetworking:SendToPeer(peer_id, "sydchSO_checkarrest", tostring(cop_id))
end

function sydchSO.CopUtils:GetCopFromId(unit_id)
    local enemies = managers.enemy:all_enemies()
    for i, unit_data in pairs(enemies) do
        if unit_data and unit_data.unit and alive(unit_data.unit) and unit_data.unit.id and tostring(unit_data.unit:id()) == unit_id then
            return unit
        end
    end
    return nil
end

function sydchSO.CopUtils:AreUnitsEnemies(unit_a, unit_b)
    if not unit_a or not unit_b or not unit_a:movement() or not unit_b:movement() then
        return false
    end

    return unit_a:movement():team().foes[unit_b:movement():team().id] and true or false
end

Hooks:Add('NetworkReceivedData', 'NetworkReceivedData_sydchSO_coputils', function(sender, messageType, data)
    if messageType ~= "sydchSO_checkarrest" then
        return
    end

    if sender ~= 1 then
        return
    end

    if not Network or Network:is_server() then
        return
    end

    local unit_id = tonumber(data)
    if not unit_id then
        return
    end

    local cop = sydchSO.CopUtils:GetCopFromId(unit_id)
    if not alive(cop) then
        return
    end

    local player_unit = managers.player and managers.player:player_unit()
    if not alive(player_unit) then
        return
    end

    local result = sydchSO.CopUtils:CheckLocalMeleeDamageArrest(managers.player:player_unit(), cop)
    if result == "counterarrest" then
        sydchSO.CopUtils:CounterArrestAttacker(managers.player:player_unit(), cop)
        return
    elseif result == "countered" then
        sydchSO.CopUtils:KnockDownAttacker(managers.player:player_unit(), cop)
        return
    elseif result == "arrested" then
        managers.player:player_unit():movement():on_cuffed()
        cop:sound():say("i03", true, false)
        return
    end
end)
